@@ -1,32 +1,74 @@
 from __future__ import annotations
 
+import re
 import warnings
 import weakref
+from _ctypes import _Pointer, _SimpleCData
 from collections import defaultdict
 from ctypes import (
-    POINTER, Structure, addressof, byref, c_buffer, c_byte, c_char, c_char_p, c_double, c_float, c_int,
-    c_short, c_ubyte, c_uint, c_ushort, cast, create_string_buffer, pointer, sizeof, string_at, Array,
+    POINTER,
+    Array,
+    Structure,
+    addressof,
+    byref,
+    c_buffer,
+    c_byte,
+    c_char,
+    c_char_p,
+    c_double,
+    c_float,
+    c_int,
+    c_short,
+    c_ubyte,
+    c_uint,
+    c_ushort,
+    cast,
+    create_string_buffer,
+    pointer,
+    sizeof,
+    string_at,
 )
-from typing import Sequence, Callable, Any, TYPE_CHECKING, Literal, Union, Type
-
-from _ctypes import _SimpleCData, _Pointer
+from typing import TYPE_CHECKING, Any, Callable, Literal, Sequence, Type, Union
 
 import pyglet
-from pyglet.gl import gl, GLException, Context, gl_info
+from pyglet.gl import Context, GLException, gl, gl_info
 from pyglet.gl.gl import (
-    glEnableVertexAttribArray, glVertexAttribPointer, glVertexAttribDivisor, glUseProgram,
-    GL_FALSE, glGetProgramiv, glGetActiveAttrib, glCreateProgram, glAttachShader, glLinkProgram, GL_LINK_STATUS,
-    GL_INFO_LOG_LENGTH, glGetProgramInfoLog, glDetachShader, glBindBufferBase, GL_UNIFORM_BUFFER,
-    glBindBuffer, GL_ARRAY_BUFFER, glMapBufferRange, GL_MAP_READ_BIT, glUnmapBuffer, GL_TRUE, glDeleteShader,
-    glDeleteProgram, glDispatchCompute, glMemoryBarrier,
+    GL_ARRAY_BUFFER,
+    GL_FALSE,
+    GL_INFO_LOG_LENGTH,
+    GL_LINK_STATUS,
+    GL_MAP_READ_BIT,
+    GL_TRUE,
+    GL_UNIFORM_BUFFER,
+    glAttachShader,
+    glBindBuffer,
+    glBindBufferBase,
+    glCreateProgram,
+    glDeleteProgram,
+    glDeleteShader,
+    glDetachShader,
+    glDispatchCompute,
+    glEnableVertexAttribArray,
+    glGetActiveAttrib,
+    glGetProgramInfoLog,
+    glGetProgramiv,
+    glLinkProgram,
+    glMapBufferRange,
+    glMemoryBarrier,
+    glUnmapBuffer,
+    glUseProgram,
+    glVertexAttribDivisor,
+    glVertexAttribIPointer,
+    glVertexAttribPointer,
 )
 from pyglet.graphics.vertexbuffer import AttributeBufferObject, BufferObject
 
 if TYPE_CHECKING:
-    from pyglet.graphics.vertexdomain import IndexedVertexList, VertexList
-    from pyglet.graphics import Group, Batch
     from _weakref import CallableProxyType
 
+    from pyglet.graphics import Batch, Group
+    from pyglet.graphics.vertexdomain import IndexedVertexList, VertexList
+
 _debug_gl_shaders = pyglet.options['debug_gl_shaders']
 
 
@@ -129,19 +171,19 @@ _uniform_setters: dict[int, tuple[GLDataType, GLFunc, GLFunc, int]] = {
     # GL_FLOAT_MAT4x3: glUniformMatrix4x3fv, glProgramUniformMatrix4x3fv,
 
     gl.GL_IMAGE_1D: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 1),
-    gl.GL_IMAGE_2D: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 2),
-    gl.GL_IMAGE_2D_RECT: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 3),
-    gl.GL_IMAGE_3D: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 3),
+    gl.GL_IMAGE_2D: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 1),
+    gl.GL_IMAGE_2D_RECT: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 1),
+    gl.GL_IMAGE_3D: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 1),
 
-    gl.GL_IMAGE_1D_ARRAY: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 2),
-    gl.GL_IMAGE_2D_ARRAY: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 3),
+    gl.GL_IMAGE_1D_ARRAY: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 1),
+    gl.GL_IMAGE_2D_ARRAY: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 1),
 
-    gl.GL_IMAGE_2D_MULTISAMPLE: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 2),
-    gl.GL_IMAGE_2D_MULTISAMPLE_ARRAY: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 3),
+    gl.GL_IMAGE_2D_MULTISAMPLE: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 1),
+    gl.GL_IMAGE_2D_MULTISAMPLE_ARRAY: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 1),
 
-    gl.GL_IMAGE_BUFFER: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 3),
+    gl.GL_IMAGE_BUFFER: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 1),
     gl.GL_IMAGE_CUBE: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 1),
-    gl.GL_IMAGE_CUBE_MAP_ARRAY: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 3),
+    gl.GL_IMAGE_CUBE_MAP_ARRAY: (gl.GLint, gl.glUniform1iv, gl.glProgramUniform1iv, 1),
 }
 
 _attribute_types: dict[int, tuple[int, str]] = {
@@ -186,9 +228,6 @@ class Attribute:
     location: int
     name: str
 
-    #: Buffer is created and attached during VertexDomain creation.
-    buffer: AttributeBufferObject | None
-
     def __init__(self, name: str, location: int, count: int, gl_type: int, normalize: bool, instance: bool) -> None:
         """Create the attribute accessor.
 
@@ -218,7 +257,9 @@ class Attribute:
 
         self.element_size = sizeof(self.c_type)
         self.stride = count * self.element_size
-        self.buffer = None
+
+        self._is_int = gl_type in (gl.GL_INT, gl.GL_SHORT, gl.GL_BYTE, gl.GL_UNSIGNED_INT,
+                                   gl.GL_UNSIGNED_SHORT, gl.GL_UNSIGNED_BYTE) and self.normalize is False
 
     def enable(self) -> None:
         """Enable the attribute."""
@@ -234,7 +275,10 @@ class Attribute:
                 Pointer offset to the currently bound buffer for this attribute.
 
         """
-        glVertexAttribPointer(self.location, self.count, self.gl_type, self.normalize, self.stride, ptr)
+        if self._is_int:
+            glVertexAttribIPointer(self.location, self.count, self.gl_type, self.stride, ptr)
+        else:
+            glVertexAttribPointer(self.location, self.count, self.gl_type, self.normalize, self.stride, ptr)
 
     def set_divisor(self) -> None:
         glVertexAttribDivisor(self.location, 1)
@@ -292,9 +336,19 @@ class _UniformArray:
     _dsa: bool
     _c_array: Array[GLDataType]
     _ptr: CTypesPointer[GLDataType]
-    _uniform: _Uniform
+    _idx_to_loc: dict[int, int]
 
-    __slots__ = ('_uniform', '_gl_type', '_gl_getter', '_gl_setter', '_is_matrix', '_dsa', '_c_array', '_ptr')
+    __slots__ = (
+        '_c_array',
+        '_dsa',
+        '_gl_getter',
+        '_gl_setter',
+        '_gl_type',
+        '_idx_to_loc',
+        '_is_matrix',
+        '_ptr',
+        '_uniform',
+    )
 
     def __init__(self, uniform: _Uniform, gl_getter: GLFunc, gl_setter: GLFunc, gl_type: GLDataType, is_matrix: bool,
                  dsa: bool) -> None:
@@ -303,6 +357,7 @@ class _UniformArray:
         self._gl_getter = gl_getter
         self._gl_setter = gl_setter
         self._is_matrix = is_matrix
+        self._idx_to_loc = {}  # Array index to uniform location mapping.
         self._dsa = dsa
 
         if self._uniform.length > 1:
@@ -312,6 +367,31 @@ class _UniformArray:
 
         self._ptr = cast(self._c_array, POINTER(gl_type))
 
+    def _get_location_for_index(self, index: int) -> int:
+        """Get the location for the array name.
+
+        It is not guaranteed that the location ID's of the uniform in the shader program will be a contiguous offset.
+
+        On MacOS, the location ID of index 0 may be 1, and then index 2 might be 5. Whereas on Windows it may be a 1:1
+        offset from 0 to index. Here, we store the location ID's of each index to ensure we are setting data on the
+        right location.
+        """
+        loc = gl.glGetUniformLocation(self._uniform.program,
+                                create_string_buffer(f"{self._uniform.name}[{index}]".encode()))
+        return loc
+
+    def _get_array_loc(self, index: int) -> int:
+        try:
+            return self._idx_to_loc[index]
+        except KeyError:
+            loc = self._idx_to_loc[index] = self._get_location_for_index(index)
+
+        if loc == -1:
+            msg = f"{self._uniform.name}[{index}] not found.\nThis may have been optimized out by the OpenGL driver if unused."
+            raise ShaderException(msg)
+
+        return loc
+
     def __len__(self) -> int:
         return self._uniform.size
 
@@ -328,8 +408,12 @@ class _UniformArray:
 
             return tuple([data for data in sliced_data])  # noqa: C416
 
-        value = self._c_array[key]
-        return tuple(value) if self._uniform.length > 1 else value
+        try:
+            value = self._c_array[key]
+            return tuple(value) if self._uniform.length > 1 else value
+        except IndexError:
+            msg = f"{self._uniform.name}[{key}] not found. This may have been optimized out by the OpenGL driver if unused."
+            raise ShaderException(msg)
 
     def __setitem__(self, key: slice | int, value: Sequence) -> None:
         if isinstance(key, slice):
@@ -366,11 +450,19 @@ class _UniformArray:
         else:
             size = self._uniform.size
 
+        location = self._get_location_for_index(offset)
+
         if self._dsa:
-            self._gl_setter(self._uniform.program, self._uniform.location + offset, size, data)
+            if self._is_matrix:
+                self._gl_setter(self._uniform.program, location, size, GL_FALSE, data)
+            else:
+                self._gl_setter(self._uniform.program, location, size, data)
         else:
             glUseProgram(self._uniform.program)
-            self._gl_setter(self._uniform.location + offset, size, data)
+            if self._is_matrix:
+                self._gl_setter(location, size, GL_FALSE, data)
+            else:
+                self._gl_setter(location, size, data)
 
     def __repr__(self) -> str:
         data = [tuple(data) if self._uniform.length > 1 else data for data in self._c_array]
@@ -389,13 +481,15 @@ class _Uniform:
     size: int
     location: int
     program: int
+    name: str
     length: int
     get: Callable[[], Array[GLDataType] | GLDataType]
     set: Callable[[float], None] | Callable[[Sequence], None]
 
-    __slots__ = 'type', 'size', 'location', 'length', 'count', 'get', 'set', 'program'
+    __slots__ = 'count', 'get', 'length', 'location', 'name', 'program', 'set', 'size', 'type'
 
-    def __init__(self, program: int, uniform_type: int, size: int, location: int, dsa: bool) -> None:
+    def __init__(self, program: int, name: str, uniform_type: int, size: int, location: int, dsa: bool) -> None:
+        self.name = name
         self.type = uniform_type
         self.size = size
         self.location = location
@@ -572,6 +666,8 @@ class _UBOBindingManager:
             msg = f"Uniform binding point: {index} is not in use."
             raise ValueError(msg)
 
+# Regular expression to detect array indices like [0], [1], etc.
+array_regex = re.compile(r"(\w+)\[(\d+)\]")
 
 class UniformBlock:
     program: CallableProxyType[Callable[..., Any] | Any] | Any
@@ -579,12 +675,12 @@ class UniformBlock:
     index: int
     size: int
     binding: int
-    uniforms: dict[int, tuple[str, GLDataType, int]]
+    uniforms: dict[int, tuple[str, GLDataType, int, int]]
     view_cls: type[Structure] | None
-    __slots__ = 'program', 'name', 'index', 'size', 'binding', 'uniforms', 'view_cls'
+    __slots__ = 'binding', 'index', 'name', 'program', 'size', 'uniform_count', 'uniforms', 'view_cls'
 
     def __init__(self, program: ShaderProgram, name: str, index: int, size: int, binding: int,
-                 uniforms: dict[int, tuple[str, GLDataType, int]]) -> None:
+                 uniforms: dict[int, tuple[str, GLDataType, int, int]], uniform_count: int) -> None:
         """Initialize a uniform block for a ShaderProgram."""
         self.program = weakref.proxy(program)
         self.name = name
@@ -592,6 +688,7 @@ class UniformBlock:
         self.size = size
         self.binding = binding
         self.uniforms = uniforms
+        self.uniform_count = uniform_count
         self.view_cls = None
 
     def create_ubo(self) -> UniformBufferObject:
@@ -633,7 +730,7 @@ class UniformBlock:
         p_id = self.program.id
         index = self.index
 
-        active_count = len(self.uniforms)
+        active_count = self.uniform_count
 
         # Query the uniform index order and each uniform's offset:
         indices = (gl.GLuint * active_count)()
@@ -653,35 +750,95 @@ class UniformBlock:
         # mat_stride = (gl.GLint * active_count)()
         # gl_types_ptr = cast(addressof(gl_types), POINTER(gl.GLint))
         # stride_ptr = cast(addressof(mat_stride), POINTER(gl.GLint))
-        # glGetActiveUniformsiv(p_id, active_count, indices, GL_UNIFORM_TYPE, gl_types_ptr)
-        # glGetActiveUniformsiv(p_id, active_count, indices, GL_UNIFORM_MATRIX_STRIDE, stride_ptr)
+        # gl.glGetActiveUniformsiv(p_id, active_count, indices, gl.GL_UNIFORM_TYPE, gl_types_ptr)
+        # gl.glGetActiveUniformsiv(p_id, active_count, indices, gl.GL_UNIFORM_MATRIX_STRIDE, stride_ptr)
 
-        view_fields = []
+        array_sizes = {}
+        dynamic_structs = {}
+        p_count = 0
+
+        rep_func = lambda s: str(dict(s._fields_))
+
+        def build_ctypes_struct(name: str, struct_dict: dict) -> type:
+            fields = []
+            for field_name, field_type in struct_dict.items():
+                if isinstance(field_type, dict):
+                    # Recursive call for nested structures
+                    element_struct = build_ctypes_struct(field_name, field_type)
+                    field_type = element_struct  # noqa: PLW2901
+                    if field_name in array_sizes and array_sizes[field_name] > 1:
+                        field_type = element_struct * array_sizes[field_name]  # noqa: PLW2901
+                else:
+                    # This handles base types like c_float_Array_2, which isn't a dict.
+                    fields.append((field_name, field_type))
+                    continue
+                fields.append((field_name, field_type))
+
+            return type(name.title(), (Structure,), {"_fields_": fields, "__repr__": rep_func})
+
+        # Build a ctypes Structure of the uniforms including arrays and nested structures.
         for i in range(active_count):
-            u_name, gl_type, length = self.uniforms[indices[i]]
-            size = offsets[i + 1] - offsets[i]
-            c_type_size = sizeof(gl_type)
-            actual_size = c_type_size * length
-            padding = size - actual_size
+            u_name, gl_type, length, u_size = self.uniforms[indices[i]]
 
-            # TODO: handle stride for multiple matrixes in the same UBO (crashes now)
-            # m_stride = mat_stride[i]
+            parts = u_name.split(".")
 
-            arg = (u_name, gl_type * length) if length > 1 else (u_name, gl_type)
-            view_fields.append(arg)
+            current_structure = dynamic_structs
+            for part_idx, part in enumerate(parts):
+                part_name = part
+                match = array_regex.match(part_name)
+                if match:  # It's an array
+                    arr_name, index = match.groups()
+                    part_name = arr_name
 
-            if padding > 0:
-                padding_bytes = padding // c_type_size
-                view_fields.append((f'_padding{i}', gl_type * padding_bytes))
+                    if part_idx != len(parts) - 1:
+                        index = int(index)  # Convert the index to an integer
+
+                        # Track array sizes for the current array name
+                        array_sizes[arr_name] = max(array_sizes.get(arr_name, 0), index + 1)
+                        if array_sizes[arr_name] > 1:
+                            break
+
+                        if arr_name not in current_structure:
+                            current_structure[arr_name] = {}
+
+                        current_structure = current_structure[arr_name]  # Move to the correct index of the array
+                        continue
+
+                # The end should be a regular attribute
+                if part_idx == len(parts) - 1:  # The last part is the actual type
+                    if u_size > 1:
+                        # If size > 1, treat as an array of type
+                        if length > 1:
+                            current_structure[part_name] = (gl_type * length) * u_size
+                        else:
+                            current_structure[part_name] = gl_type * u_size
+                    else:
+                        if length > 1:
+                            current_structure[part_name] = gl_type * length
+                        else:
+                            current_structure[part_name] = gl_type
+
+                    offset_size = offsets[i + 1] - offsets[i]
+                    c_type_size = sizeof(current_structure[part_name])
+                    padding = offset_size - c_type_size
+
+                    # TODO: Cannot get a different stride on my hardware. Needs testing.
+                    # is_matrix = gl_types[i] in _gl_matrices
+                    # if is_matrix:
+                    #     stride_padding = (mat_stride[i] // 4) * 4 - offset_size
+                    #     if stride_padding > 0:
+                    #         view_fields.append((f'_matrix_stride{i}', c_byte * stride_padding))
+
+                    if padding > 0:
+                        current_structure[f'_padding{p_count}'] = c_byte * padding
+                        p_count += 1
+                else:
+                    if part_name not in current_structure:
+                        current_structure[part_name] = {}
+                    current_structure = current_structure[part_name]  # Drill down into nested structures
 
         # Custom ctypes Structure for Uniform access:
-        class View(Structure):
-            _fields_ = view_fields
-
-            def __repr__(self) -> str:
-                return str(dict(self._fields_))
-
-        return View
+        return build_ctypes_struct('View', dynamic_structs)
 
     def _actual_binding_point(self) -> int:
         """Queries OpenGL to find what the bind point currently is."""
@@ -700,7 +857,7 @@ class UniformBufferObject:
     _view_ptr: CTypesPointer[Structure]
     binding: int
     buffer: BufferObject
-    __slots__ = 'buffer', 'view', '_view_ptr', 'binding'
+    __slots__ = '_view_ptr', 'binding', 'buffer', 'view'
 
     def __init__(self, view_class: type[Structure], buffer_size: int, binding: int) -> None:
         """Initialize the Uniform Buffer Object with the specified Structure."""
@@ -845,9 +1002,9 @@ def _introspect_uniforms(program_id: int, have_dsa: bool) -> dict[str, _Uniform]
     for index in range(_get_number(program_id, gl.GL_ACTIVE_UNIFORMS)):
         u_name, u_type, u_size = _query_uniform(program_id, index)
 
-        # Multidimensional arrays cannot be fully inspected via OpenGL calls.
+        # Multidimensional arrays cannot be fully inspected via OpenGL calls and compile errors with 3.3.
         array_count = u_name.count("[0]")
-        if array_count > 1:
+        if array_count > 1 and u_name.count("[0][0]") != 0:
             msg = "Multidimensional arrays are not currently supported."
             raise ShaderException(msg)
 
@@ -856,11 +1013,11 @@ def _introspect_uniforms(program_id: int, have_dsa: bool) -> dict[str, _Uniform]
             continue
 
         # Strip [0] from array name for a more user-friendly name.
-        if array_count == 1:
+        if array_count != 0:
             u_name = u_name.strip('[0]')
 
         assert u_name not in uniforms, f"{u_name} exists twice in the shader. Possible name clash with an array."
-        uniforms[u_name] = _Uniform(program_id, u_type, u_size, loc, have_dsa)
+        uniforms[u_name] = _Uniform(program_id, u_name, u_type, u_size, loc, have_dsa)
 
     if _debug_gl_shaders:
         for uniform in uniforms.values():
@@ -901,7 +1058,7 @@ def _introspect_uniform_blocks(program: ShaderProgram | ComputeShaderProgram) ->
         indices_ptr = cast(addressof(indices), POINTER(gl.GLint))
         gl.glGetActiveUniformBlockiv(program_id, index, gl.GL_UNIFORM_BLOCK_ACTIVE_UNIFORM_INDICES, indices_ptr)
 
-        uniforms = {}
+        uniforms: dict[int, tuple[str, GLDataType, int, int]] = {}
 
         if not hasattr(pyglet.gl.current_context, "ubo_manager"):
             pyglet.gl.current_context.ubo_manager = _UBOBindingManager()
@@ -911,14 +1068,16 @@ def _introspect_uniform_blocks(program: ShaderProgram | ComputeShaderProgram) ->
         for block_uniform_index in indices:
             uniform_name, u_type, u_size = _query_uniform(program_id, block_uniform_index)
 
-            # Separate uniform name from block name (Only if instance name is provided on the Uniform Block)
-            try:
-                _, uniform_name = uniform_name.split(".")
-            except ValueError:
-                pass
+            # Remove block name.
+            if uniform_name.startswith(f"{name}."):
+                uniform_name = uniform_name[len(name) + 1:]  # Strip 'block_name.' part
+
+            if uniform_name.count("[0][0]") > 0:
+                msg = "Multidimensional arrays are not currently supported."
+                raise ShaderException(msg)
 
             gl_type, _, _, length = _uniform_setters[u_type]
-            uniforms[block_uniform_index] = (uniform_name, gl_type, length)
+            uniforms[block_uniform_index] = (uniform_name, gl_type, length, u_size)
 
         binding_index = binding.value
         if pyglet.options.shader_bind_management:
@@ -938,7 +1097,8 @@ def _introspect_uniform_blocks(program: ShaderProgram | ComputeShaderProgram) ->
                     warnings.warn(msg)
                 manager.add_explicit_binding(program, name, binding.value)
 
-        uniform_blocks[name] = UniformBlock(program, name, index, block_data_size.value, binding_index, uniforms)
+        uniform_blocks[name] = UniformBlock(program, name, index, block_data_size.value, binding_index, uniforms,
+                                            len(indices))
 
         if _debug_gl_shaders:
             for block in uniform_blocks.values():
@@ -1128,7 +1288,7 @@ class ShaderProgram:
     _uniforms: dict[str, _Uniform]
     _uniform_blocks: dict[str, UniformBlock]
 
-    __slots__ = '_id', '_context', '_attributes', '_uniforms', '_uniform_blocks', '__weakref__'
+    __slots__ = '__weakref__', '_attributes', '_context', '_id', '_uniform_blocks', '_uniforms'
 
     def __init__(self, *shaders: Shader) -> None:
         """Initialize the ShaderProgram using at least two Shader instances."""
@@ -1263,32 +1423,38 @@ class ShaderProgram:
                 if isinstance(fmt, tuple):
                     fmt, array = fmt  # noqa: PLW2901
                     initial_arrays.append((name, array))
-                attributes[name] = {
-                    **attributes[name], 'format': fmt,
-                    'instance': name in instances if instances else False,
-                }
+                attributes[name] = {**attributes[name], 'format': fmt, 'instance': name in instances if instances else False}
             except KeyError:  # noqa: PERF203
+                if _debug_gl_shaders:
+                    msg = (f"The attribute `{name}` was not found in the Shader Program.\n"
+                           f"Please check the spelling, or it may have been optimized out by the OpenGL driver.\n"
+                           f"Valid names: {list(attributes)}")
+                    warnings.warn(msg)
+                continue
+
+        if _debug_gl_shaders:
+            if missing_data := [key for key in attributes if key not in data]:
                 msg = (
-                    f"An attribute with the name `{name}` was not found. Please "
-                    f"check the spelling.\nIf the attribute is not in use in the "
-                    f"program, it may have been optimized out by the OpenGL driver.\n"
-                    f"Valid names: \n{list(attributes)}"
+                    f"No data was supplied for the following found attributes: `{missing_data}`.\n"
                 )
-                raise ShaderException(msg)  # noqa: B904
+                warnings.warn(msg)
 
         batch = batch or pyglet.graphics.get_default_batch()
-        domain = batch.get_domain(indexed, instanced, mode, group, self, attributes)
+        group = group or pyglet.graphics.ShaderGroup(program=self)
+        domain = batch.get_domain(indexed, instanced, mode, group, attributes)
 
         # Create vertex list and initialize
         if indexed:
             vlist = domain.create(count, len(indices))
-            start = vlist.start
-            vlist.indices = [i + start for i in indices]
+            vlist.indices = indices
         else:
             vlist = domain.create(count)
 
         for name, array in initial_arrays:
-            vlist.set_attribute_data(name, array)
+            try:
+                vlist.set_attribute_data(name, array)
+            except KeyError:  # noqa: PERF203
+                continue
 
         if instanced:
             vlist.instanced = True
