@@ -1,16 +1,13 @@
-from __future__ import division
-from builtins import range
 from tests import mock
 import random
-from tests.base.future_test import FutureTestCase
+import unittest
 
+from pyglet.gl import GL_TEXTURE_2D
 from pyglet.media.player import Player, PlayerGroup
 from pyglet.media.codecs.base import AudioFormat, VideoFormat, Source
 
-# pyglet.options['debug_media'] = True
 
-
-class PlayerTestCase(FutureTestCase):
+class PlayerTestCase(unittest.TestCase):
     # Default values to use
     audio_format_1 = AudioFormat(1, 8, 11025)
     audio_format_2 = AudioFormat(2, 8, 11025)
@@ -51,8 +48,9 @@ class PlayerTestCase(FutureTestCase):
         mock_source = mock.MagicMock(spec=Source)
         type(mock_source).audio_format = mock.PropertyMock(return_value=audio_format)
         type(mock_source).video_format = mock.PropertyMock(return_value=video_format)
-        type(mock_source._get_queue_source.return_value).audio_format = mock.PropertyMock(return_value=audio_format)
-        type(mock_source._get_queue_source.return_value).video_format = mock.PropertyMock(return_value=video_format)
+        type(mock_source.get_queue_source.return_value).audio_format = mock.PropertyMock(return_value=audio_format)
+        type(mock_source.get_queue_source.return_value).video_format = mock.PropertyMock(return_value=video_format)
+        type(mock_source.get_queue_source.return_value).duration = mock.PropertyMock(return_value=None)
         if video_format:
             mock_source.video_format.frame_rate = 30
         return mock_source
@@ -86,7 +84,7 @@ class PlayerTestCase(FutureTestCase):
 
     def _assert_playing(self, playing, current_source=None):
         self.assertEqual(self.player.playing, playing)
-        queued_source = (current_source._get_queue_source.return_value
+        queued_source = (current_source.get_queue_source.return_value
                          if current_source is not None
                          else None)
         self.assertIs(self.player.source, queued_source)
@@ -111,7 +109,7 @@ class PlayerTestCase(FutureTestCase):
     # def assert_in_current_play_list(self, *sources):
     #     self.assertIsNotNone(self.current_play_list, msg='No previous call to create driver player')
 
-    #     queue_sources = deque(source._get_queue_source.return_value for source in sources)
+    #     queue_sources = deque(source.get_queue_source.return_value for source in sources)
     #     self.assertSequenceEqual(self.current_play_list._sources, queue_sources)
 
     def assert_driver_player_destroyed(self):
@@ -136,7 +134,7 @@ class PlayerTestCase(FutureTestCase):
         source.get_queue_source.return_value.seek.assert_called_once_with(time)
 
     def assert_new_texture_created(self, video_format):
-        self.mock_texture_create.assert_called_once_with(video_format.width, video_format.height, rectangle=True)
+        self.mock_texture_create.assert_called_once_with(video_format.width, video_format.height, GL_TEXTURE_2D)
 
     def assert_no_new_texture_created(self):
         self.assertFalse(self.mock_texture_create.called)
@@ -154,7 +152,7 @@ class PlayerTestCase(FutureTestCase):
         self.mock_clock.unschedule.assert_called_with(self.player.update_texture)
 
     def pretend_player_at_time(self, t):
-        self.player._mclock.set_time(t)
+        self.player._timer.set_time(t)
 
     def pretend_silent_driver_player_at_time(self, t):
         self.mock_silent_audio_driver_player.get_time.return_value = t
@@ -193,7 +191,7 @@ class PlayerTestCase(FutureTestCase):
 
     def test_queue_multiple_audio_sources_different_format_and_play_and_skip(self):
         """Queue multiple audio sources having different formats and start playing. Different
-        formats should be played by seperate driver players."""
+        formats should be played by separate driver players."""
         mock_source1 = self.create_mock_source(self.audio_format_1, None)
         mock_source2 = self.create_mock_source(self.audio_format_2, None)
         mock_source3 = self.create_mock_source(self.audio_format_3, None)
@@ -358,9 +356,10 @@ class PlayerTestCase(FutureTestCase):
 
         self.reset_mocks()
         self.player.play()
+        self.assertGreaterEqual(self.player.time, 0.5)
         self.assertAlmostEqual(
-            self.player.time, 0.5, places=2,
-            msg='While playing, player should return time from driver player'
+            self.player.time, 0.5, delta=0.01,
+            msg='Player time should have advanced during pause'
         )
         self.assert_driver_player_started()
         self.assert_no_new_driver_player_created()
@@ -691,7 +690,7 @@ class PlayerTestCase(FutureTestCase):
         self.player.play()
 
 
-class PlayerGroupTestCase(FutureTestCase):
+class PlayerGroupTestCase(unittest.TestCase):
     def create_mock_player(self, has_audio=True):
         player = mock.MagicMock()
         if has_audio:
@@ -714,16 +713,13 @@ class PlayerGroupTestCase(FutureTestCase):
             audio_player = player._audio_player
             audio_players.append(audio_player)
             if call_args is not None:
-                self.assertFalse(audio_player._play_group.called,
-                                 msg='Only one player should be used to start the group')
+                self.assertFalse(audio_player._play_group.called, msg='Only one player should be used to start the group')
             elif audio_player._play_group.called:
                 call_args = audio_player._play_group.call_args
 
-        self.assertIsNotNone(call_args,
-                             msg='No player was used to start all audio players.')
+        self.assertIsNotNone(call_args, msg='No player was used to start all audio players.')
         started_players = call_args[0][0]
-        self.assertCountEqual(started_players, audio_players,
-                              msg='Not all players with audio players were started')
+        self.assertCountEqual(started_players, audio_players, msg='Not all players with audio players were started')
 
     def assert_players_stopped(self, *players):
         for player in players:
